/* DO NOT EDIT - This file generated automatically by api_hw_select_init_h.py script */

/*
 * Copyright (C) 2012 Intel Corporation
 * Copyright (C) 2021 Advanced Micro Devices, Inc.
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.  IN NO EVENT SHALL
 * AUTHORS,
 * AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

SET_Vertex2d(tab, NAME(Vertex2d));
SET_Vertex2dv(tab, NAME(Vertex2dv));
SET_Vertex2f(tab, NAME(Vertex2f));
SET_Vertex2fv(tab, NAME(Vertex2fv));
SET_Vertex2i(tab, NAME(Vertex2i));
SET_Vertex2iv(tab, NAME(Vertex2iv));
SET_Vertex2s(tab, NAME(Vertex2s));
SET_Vertex2sv(tab, NAME(Vertex2sv));
SET_Vertex3d(tab, NAME(Vertex3d));
SET_Vertex3dv(tab, NAME(Vertex3dv));
SET_Vertex3f(tab, NAME(Vertex3f));
SET_Vertex3fv(tab, NAME(Vertex3fv));
SET_Vertex3i(tab, NAME(Vertex3i));
SET_Vertex3iv(tab, NAME(Vertex3iv));
SET_Vertex3s(tab, NAME(Vertex3s));
SET_Vertex3sv(tab, NAME(Vertex3sv));
SET_Vertex4d(tab, NAME(Vertex4d));
SET_Vertex4dv(tab, NAME(Vertex4dv));
SET_Vertex4f(tab, NAME(Vertex4f));
SET_Vertex4fv(tab, NAME(Vertex4fv));
SET_Vertex4i(tab, NAME(Vertex4i));
SET_Vertex4iv(tab, NAME(Vertex4iv));
SET_Vertex4s(tab, NAME(Vertex4s));
SET_Vertex4sv(tab, NAME(Vertex4sv));
SET_VertexAttrib1d(tab, NAME(VertexAttrib1d));
SET_VertexAttrib1dv(tab, NAME(VertexAttrib1dv));
SET_VertexAttrib1fARB(tab, NAME(VertexAttrib1fARB));
SET_VertexAttrib1fvARB(tab, NAME(VertexAttrib1fvARB));
SET_VertexAttrib1s(tab, NAME(VertexAttrib1s));
SET_VertexAttrib1sv(tab, NAME(VertexAttrib1sv));
SET_VertexAttrib2d(tab, NAME(VertexAttrib2d));
SET_VertexAttrib2dv(tab, NAME(VertexAttrib2dv));
SET_VertexAttrib2fARB(tab, NAME(VertexAttrib2fARB));
SET_VertexAttrib2fvARB(tab, NAME(VertexAttrib2fvARB));
SET_VertexAttrib2s(tab, NAME(VertexAttrib2s));
SET_VertexAttrib2sv(tab, NAME(VertexAttrib2sv));
SET_VertexAttrib3d(tab, NAME(VertexAttrib3d));
SET_VertexAttrib3dv(tab, NAME(VertexAttrib3dv));
SET_VertexAttrib3fARB(tab, NAME(VertexAttrib3fARB));
SET_VertexAttrib3fvARB(tab, NAME(VertexAttrib3fvARB));
SET_VertexAttrib3s(tab, NAME(VertexAttrib3s));
SET_VertexAttrib3sv(tab, NAME(VertexAttrib3sv));
SET_VertexAttrib4Nbv(tab, NAME(VertexAttrib4Nbv));
SET_VertexAttrib4Niv(tab, NAME(VertexAttrib4Niv));
SET_VertexAttrib4Nsv(tab, NAME(VertexAttrib4Nsv));
SET_VertexAttrib4Nub(tab, NAME(VertexAttrib4Nub));
SET_VertexAttrib4Nubv(tab, NAME(VertexAttrib4Nubv));
SET_VertexAttrib4Nuiv(tab, NAME(VertexAttrib4Nuiv));
SET_VertexAttrib4Nusv(tab, NAME(VertexAttrib4Nusv));
SET_VertexAttrib4bv(tab, NAME(VertexAttrib4bv));
SET_VertexAttrib4d(tab, NAME(VertexAttrib4d));
SET_VertexAttrib4dv(tab, NAME(VertexAttrib4dv));
SET_VertexAttrib4fARB(tab, NAME(VertexAttrib4fARB));
SET_VertexAttrib4fvARB(tab, NAME(VertexAttrib4fvARB));
SET_VertexAttrib4iv(tab, NAME(VertexAttrib4iv));
SET_VertexAttrib4s(tab, NAME(VertexAttrib4s));
SET_VertexAttrib4sv(tab, NAME(VertexAttrib4sv));
SET_VertexAttrib4ubv(tab, NAME(VertexAttrib4ubv));
SET_VertexAttrib4uiv(tab, NAME(VertexAttrib4uiv));
SET_VertexAttrib4usv(tab, NAME(VertexAttrib4usv));
SET_VertexP2ui(tab, NAME(VertexP2ui));
SET_VertexP3ui(tab, NAME(VertexP3ui));
SET_VertexP4ui(tab, NAME(VertexP4ui));
SET_VertexP2uiv(tab, NAME(VertexP2uiv));
SET_VertexP3uiv(tab, NAME(VertexP3uiv));
SET_VertexP4uiv(tab, NAME(VertexP4uiv));
SET_VertexAttribP1ui(tab, NAME(VertexAttribP1ui));
SET_VertexAttribP2ui(tab, NAME(VertexAttribP2ui));
SET_VertexAttribP3ui(tab, NAME(VertexAttribP3ui));
SET_VertexAttribP4ui(tab, NAME(VertexAttribP4ui));
SET_VertexAttribP1uiv(tab, NAME(VertexAttribP1uiv));
SET_VertexAttribP2uiv(tab, NAME(VertexAttribP2uiv));
SET_VertexAttribP3uiv(tab, NAME(VertexAttribP3uiv));
SET_VertexAttribP4uiv(tab, NAME(VertexAttribP4uiv));
SET_VertexAttribL1d(tab, NAME(VertexAttribL1d));
SET_VertexAttribL2d(tab, NAME(VertexAttribL2d));
SET_VertexAttribL3d(tab, NAME(VertexAttribL3d));
SET_VertexAttribL4d(tab, NAME(VertexAttribL4d));
SET_VertexAttribL1dv(tab, NAME(VertexAttribL1dv));
SET_VertexAttribL2dv(tab, NAME(VertexAttribL2dv));
SET_VertexAttribL3dv(tab, NAME(VertexAttribL3dv));
SET_VertexAttribL4dv(tab, NAME(VertexAttribL4dv));
SET_VertexAttribL1ui64ARB(tab, NAME(VertexAttribL1ui64ARB));
SET_VertexAttribL1ui64vARB(tab, NAME(VertexAttribL1ui64vARB));
SET_VertexAttrib1sNV(tab, NAME(VertexAttrib1sNV));
SET_VertexAttrib1svNV(tab, NAME(VertexAttrib1svNV));
SET_VertexAttrib2sNV(tab, NAME(VertexAttrib2sNV));
SET_VertexAttrib2svNV(tab, NAME(VertexAttrib2svNV));
SET_VertexAttrib3sNV(tab, NAME(VertexAttrib3sNV));
SET_VertexAttrib3svNV(tab, NAME(VertexAttrib3svNV));
SET_VertexAttrib4sNV(tab, NAME(VertexAttrib4sNV));
SET_VertexAttrib4svNV(tab, NAME(VertexAttrib4svNV));
SET_VertexAttrib1fNV(tab, NAME(VertexAttrib1fNV));
SET_VertexAttrib1fvNV(tab, NAME(VertexAttrib1fvNV));
SET_VertexAttrib2fNV(tab, NAME(VertexAttrib2fNV));
SET_VertexAttrib2fvNV(tab, NAME(VertexAttrib2fvNV));
SET_VertexAttrib3fNV(tab, NAME(VertexAttrib3fNV));
SET_VertexAttrib3fvNV(tab, NAME(VertexAttrib3fvNV));
SET_VertexAttrib4fNV(tab, NAME(VertexAttrib4fNV));
SET_VertexAttrib4fvNV(tab, NAME(VertexAttrib4fvNV));
SET_VertexAttrib1dNV(tab, NAME(VertexAttrib1dNV));
SET_VertexAttrib1dvNV(tab, NAME(VertexAttrib1dvNV));
SET_VertexAttrib2dNV(tab, NAME(VertexAttrib2dNV));
SET_VertexAttrib2dvNV(tab, NAME(VertexAttrib2dvNV));
SET_VertexAttrib3dNV(tab, NAME(VertexAttrib3dNV));
SET_VertexAttrib3dvNV(tab, NAME(VertexAttrib3dvNV));
SET_VertexAttrib4dNV(tab, NAME(VertexAttrib4dNV));
SET_VertexAttrib4dvNV(tab, NAME(VertexAttrib4dvNV));
SET_VertexAttrib4ubNV(tab, NAME(VertexAttrib4ubNV));
SET_VertexAttrib4ubvNV(tab, NAME(VertexAttrib4ubvNV));
SET_VertexAttribs1svNV(tab, NAME(VertexAttribs1svNV));
SET_VertexAttribs2svNV(tab, NAME(VertexAttribs2svNV));
SET_VertexAttribs3svNV(tab, NAME(VertexAttribs3svNV));
SET_VertexAttribs4svNV(tab, NAME(VertexAttribs4svNV));
SET_VertexAttribs1fvNV(tab, NAME(VertexAttribs1fvNV));
SET_VertexAttribs2fvNV(tab, NAME(VertexAttribs2fvNV));
SET_VertexAttribs3fvNV(tab, NAME(VertexAttribs3fvNV));
SET_VertexAttribs4fvNV(tab, NAME(VertexAttribs4fvNV));
SET_VertexAttribs1dvNV(tab, NAME(VertexAttribs1dvNV));
SET_VertexAttribs2dvNV(tab, NAME(VertexAttribs2dvNV));
SET_VertexAttribs3dvNV(tab, NAME(VertexAttribs3dvNV));
SET_VertexAttribs4dvNV(tab, NAME(VertexAttribs4dvNV));
SET_VertexAttribs4ubvNV(tab, NAME(VertexAttribs4ubvNV));
SET_VertexAttribI1iEXT(tab, NAME(VertexAttribI1iEXT));
SET_VertexAttribI2iEXT(tab, NAME(VertexAttribI2iEXT));
SET_VertexAttribI3iEXT(tab, NAME(VertexAttribI3iEXT));
SET_VertexAttribI4iEXT(tab, NAME(VertexAttribI4iEXT));
SET_VertexAttribI1uiEXT(tab, NAME(VertexAttribI1uiEXT));
SET_VertexAttribI2uiEXT(tab, NAME(VertexAttribI2uiEXT));
SET_VertexAttribI3uiEXT(tab, NAME(VertexAttribI3uiEXT));
SET_VertexAttribI4uiEXT(tab, NAME(VertexAttribI4uiEXT));
SET_VertexAttribI1iv(tab, NAME(VertexAttribI1iv));
SET_VertexAttribI2ivEXT(tab, NAME(VertexAttribI2ivEXT));
SET_VertexAttribI3ivEXT(tab, NAME(VertexAttribI3ivEXT));
SET_VertexAttribI4ivEXT(tab, NAME(VertexAttribI4ivEXT));
SET_VertexAttribI1uiv(tab, NAME(VertexAttribI1uiv));
SET_VertexAttribI2uivEXT(tab, NAME(VertexAttribI2uivEXT));
SET_VertexAttribI3uivEXT(tab, NAME(VertexAttribI3uivEXT));
SET_VertexAttribI4uivEXT(tab, NAME(VertexAttribI4uivEXT));
SET_VertexAttribI4bv(tab, NAME(VertexAttribI4bv));
SET_VertexAttribI4sv(tab, NAME(VertexAttribI4sv));
SET_VertexAttribI4ubv(tab, NAME(VertexAttribI4ubv));
SET_VertexAttribI4usv(tab, NAME(VertexAttribI4usv));
SET_Vertex2hNV(tab, NAME(Vertex2hNV));
SET_Vertex2hvNV(tab, NAME(Vertex2hvNV));
SET_Vertex3hNV(tab, NAME(Vertex3hNV));
SET_Vertex3hvNV(tab, NAME(Vertex3hvNV));
SET_Vertex4hNV(tab, NAME(Vertex4hNV));
SET_Vertex4hvNV(tab, NAME(Vertex4hvNV));
SET_VertexAttrib1hNV(tab, NAME(VertexAttrib1hNV));
SET_VertexAttrib1hvNV(tab, NAME(VertexAttrib1hvNV));
SET_VertexAttrib2hNV(tab, NAME(VertexAttrib2hNV));
SET_VertexAttrib2hvNV(tab, NAME(VertexAttrib2hvNV));
SET_VertexAttrib3hNV(tab, NAME(VertexAttrib3hNV));
SET_VertexAttrib3hvNV(tab, NAME(VertexAttrib3hvNV));
SET_VertexAttrib4hNV(tab, NAME(VertexAttrib4hNV));
SET_VertexAttrib4hvNV(tab, NAME(VertexAttrib4hvNV));
SET_VertexAttribs1hvNV(tab, NAME(VertexAttribs1hvNV));
SET_VertexAttribs2hvNV(tab, NAME(VertexAttribs2hvNV));
SET_VertexAttribs3hvNV(tab, NAME(VertexAttribs3hvNV));
SET_VertexAttribs4hvNV(tab, NAME(VertexAttribs4hvNV));
