/*
 * File generated with ../mesa-22.2.3/src/gallium/auxiliary/driver_trace/enums2names.py, please do not edit manually.
 */
#ifndef TR_UTIL_H
#define TR_UTIL_H


#include "pipe/p_defines.h"


#ifdef __cplusplus
extern "C" {
#endif

const char *
tr_util_pipe_texture_target_name(enum pipe_texture_target value);
const char *
tr_util_pipe_shader_type_name(enum pipe_shader_type value);
const char *
tr_util_pipe_shader_cap_name(enum pipe_shader_cap value);
const char *
tr_util_pipe_shader_ir_name(enum pipe_shader_ir value);
const char *
tr_util_pipe_cap_name(enum pipe_cap value);
const char *
tr_util_pipe_capf_name(enum pipe_capf value);
const char *
tr_util_pipe_compute_cap_name(enum pipe_compute_cap value);
const char *
tr_util_pipe_resource_param_name(enum pipe_resource_param value);
const char *
tr_util_pipe_fd_type_name(enum pipe_fd_type value);


#ifdef __cplusplus
}
#endif

#endif /* TR_UTIL_H */        
