/* DO NOT EDIT - This file is generated automatically by vtn_generator_ids.py script */

/*
 * Copyright (C) 2020 Valve Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 */

enum vtn_generator {
   vtn_generator_llvm_spirv_translator = 6,
   vtn_generator_spirv_tools_assembler = 7,
   vtn_generator_glslang_reference_front_end = 8,
   vtn_generator_shaderc_over_glslang = 13,
   vtn_generator_spiregg = 14,
   vtn_generator_rspirv = 15,
   vtn_generator_mesair_spirv_translator = 16,
   vtn_generator_spirv_tools_linker = 17,
   vtn_generator_vkd3d_shader_compiler = 18,
   vtn_generator_clay_shader_compiler = 19,
   vtn_generator_whlsl_shader_translator = 20,
   vtn_generator_clspv = 21,
   vtn_generator_mlir_spirv_serializer = 22,
   vtn_generator_tint_compiler = 23,
   vtn_generator_angle_shader_compiler = 24,
   vtn_generator_messiah_shader_compiler = 25,
   vtn_generator_xenia_emulator_microcode_translator = 26,
   vtn_generator_rust_gpu_compiler_backend = 27,
   vtn_generator_naga = 28,
   vtn_generator_msp_shader_compiler = 29,
   vtn_generator_spvgentwo_spirv_ir_tools = 30,
   vtn_generator_skia_sksl = 31,
   vtn_generator_spirv_beehive_toolkit = 32,
   vtn_generator_max = 0xffff,
};
