/* DO NOT EDIT - This file generated automatically by api_save_init_h.py script */

/*
 * Copyright (C) 2012 Intel Corporation
 * Copyright (C) 2021 Advanced Micro Devices, Inc.
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.  IN NO EVENT SHALL
 * AUTHORS,
 * AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

SET_NewList(table, save_NewList);
SET_ListBase(table, save_ListBase);
SET_Bitmap(table, save_Bitmap);
SET_RasterPos2d(table, save_RasterPos2d);
SET_RasterPos2dv(table, save_RasterPos2dv);
SET_RasterPos2f(table, save_RasterPos2f);
SET_RasterPos2fv(table, save_RasterPos2fv);
SET_RasterPos2i(table, save_RasterPos2i);
SET_RasterPos2iv(table, save_RasterPos2iv);
SET_RasterPos2s(table, save_RasterPos2s);
SET_RasterPos2sv(table, save_RasterPos2sv);
SET_RasterPos3d(table, save_RasterPos3d);
SET_RasterPos3dv(table, save_RasterPos3dv);
SET_RasterPos3f(table, save_RasterPos3f);
SET_RasterPos3fv(table, save_RasterPos3fv);
SET_RasterPos3i(table, save_RasterPos3i);
SET_RasterPos3iv(table, save_RasterPos3iv);
SET_RasterPos3s(table, save_RasterPos3s);
SET_RasterPos3sv(table, save_RasterPos3sv);
SET_RasterPos4d(table, save_RasterPos4d);
SET_RasterPos4dv(table, save_RasterPos4dv);
SET_RasterPos4f(table, save_RasterPos4f);
SET_RasterPos4fv(table, save_RasterPos4fv);
SET_RasterPos4i(table, save_RasterPos4i);
SET_RasterPos4iv(table, save_RasterPos4iv);
SET_RasterPos4s(table, save_RasterPos4s);
SET_RasterPos4sv(table, save_RasterPos4sv);
SET_Rectdv(table, save_Rectdv);
SET_Rectf(table, save_Rectf);
SET_Rectfv(table, save_Rectfv);
SET_Recti(table, save_Recti);
SET_Rectiv(table, save_Rectiv);
SET_Rects(table, save_Rects);
SET_Rectsv(table, save_Rectsv);
SET_ClipPlane(table, save_ClipPlane);
SET_ColorMaterial(table, save_ColorMaterial);
SET_CullFace(table, save_CullFace);
SET_Fogf(table, save_Fogf);
SET_Fogfv(table, save_Fogfv);
SET_Fogi(table, save_Fogi);
SET_Fogiv(table, save_Fogiv);
SET_FrontFace(table, save_FrontFace);
SET_Hint(table, save_Hint);
SET_Lightf(table, save_Lightf);
SET_Lightfv(table, save_Lightfv);
SET_Lighti(table, save_Lighti);
SET_Lightiv(table, save_Lightiv);
SET_LightModelf(table, save_LightModelf);
SET_LightModelfv(table, save_LightModelfv);
SET_LightModeli(table, save_LightModeli);
SET_LightModeliv(table, save_LightModeliv);
SET_LineStipple(table, save_LineStipple);
SET_LineWidth(table, save_LineWidth);
SET_PointSize(table, save_PointSize);
SET_PolygonMode(table, save_PolygonMode);
SET_PolygonStipple(table, save_PolygonStipple);
SET_Scissor(table, save_Scissor);
SET_ShadeModel(table, save_ShadeModel);
SET_TexParameterf(table, save_TexParameterf);
SET_TexParameterfv(table, save_TexParameterfv);
SET_TexParameteri(table, save_TexParameteri);
SET_TexParameteriv(table, save_TexParameteriv);
SET_TexImage1D(table, save_TexImage1D);
SET_TexImage2D(table, save_TexImage2D);
SET_TexEnvf(table, save_TexEnvf);
SET_TexEnvfv(table, save_TexEnvfv);
SET_TexEnvi(table, save_TexEnvi);
SET_TexEnviv(table, save_TexEnviv);
SET_TexGend(table, save_TexGend);
SET_TexGendv(table, save_TexGendv);
SET_TexGenf(table, save_TexGenf);
SET_TexGenfv(table, save_TexGenfv);
SET_TexGeni(table, save_TexGeni);
SET_TexGeniv(table, save_TexGeniv);
SET_InitNames(table, save_InitNames);
SET_LoadName(table, save_LoadName);
SET_PassThrough(table, save_PassThrough);
SET_PopName(table, save_PopName);
SET_PushName(table, save_PushName);
SET_DrawBuffer(table, save_DrawBuffer);
SET_Clear(table, save_Clear);
SET_ClearAccum(table, save_ClearAccum);
SET_ClearIndex(table, save_ClearIndex);
SET_ClearColor(table, save_ClearColor);
SET_ClearStencil(table, save_ClearStencil);
SET_ClearDepth(table, save_ClearDepth);
SET_StencilMask(table, save_StencilMask);
SET_ColorMask(table, save_ColorMask);
SET_DepthMask(table, save_DepthMask);
SET_IndexMask(table, save_IndexMask);
SET_Accum(table, save_Accum);
SET_Disable(table, save_Disable);
SET_Enable(table, save_Enable);
SET_PopAttrib(table, save_PopAttrib);
SET_PushAttrib(table, save_PushAttrib);
SET_Map1d(table, save_Map1d);
SET_Map1f(table, save_Map1f);
SET_Map2d(table, save_Map2d);
SET_Map2f(table, save_Map2f);
SET_MapGrid1d(table, save_MapGrid1d);
SET_MapGrid1f(table, save_MapGrid1f);
SET_MapGrid2d(table, save_MapGrid2d);
SET_MapGrid2f(table, save_MapGrid2f);
SET_EvalMesh1(table, save_EvalMesh1);
SET_EvalMesh2(table, save_EvalMesh2);
SET_AlphaFunc(table, save_AlphaFunc);
SET_BlendFunc(table, save_BlendFunc);
SET_LogicOp(table, save_LogicOp);
SET_StencilFunc(table, save_StencilFunc);
SET_StencilOp(table, save_StencilOp);
SET_DepthFunc(table, save_DepthFunc);
SET_PixelZoom(table, save_PixelZoom);
SET_PixelTransferf(table, save_PixelTransferf);
SET_PixelTransferi(table, save_PixelTransferi);
SET_PixelMapfv(table, save_PixelMapfv);
SET_PixelMapuiv(table, save_PixelMapuiv);
SET_PixelMapusv(table, save_PixelMapusv);
SET_ReadBuffer(table, save_ReadBuffer);
SET_CopyPixels(table, save_CopyPixels);
SET_DrawPixels(table, save_DrawPixels);
SET_DepthRange(table, save_DepthRange);
SET_Frustum(table, save_Frustum);
SET_LoadIdentity(table, save_LoadIdentity);
SET_LoadMatrixf(table, save_LoadMatrixf);
SET_LoadMatrixd(table, save_LoadMatrixd);
SET_MatrixMode(table, save_MatrixMode);
SET_MultMatrixf(table, save_MultMatrixf);
SET_MultMatrixd(table, save_MultMatrixd);
SET_Ortho(table, save_Ortho);
SET_PopMatrix(table, save_PopMatrix);
SET_PushMatrix(table, save_PushMatrix);
SET_Rotated(table, save_Rotated);
SET_Rotatef(table, save_Rotatef);
SET_Scaled(table, save_Scaled);
SET_Scalef(table, save_Scalef);
SET_Translated(table, save_Translated);
SET_Translatef(table, save_Translatef);
SET_Viewport(table, save_Viewport);
SET_DrawArrays(table, save_DrawArrays);
SET_DrawElements(table, save_DrawElements);
SET_PolygonOffset(table, save_PolygonOffset);
SET_CopyTexImage1D(table, save_CopyTexImage1D);
SET_CopyTexImage2D(table, save_CopyTexImage2D);
SET_CopyTexSubImage1D(table, save_CopyTexSubImage1D);
SET_CopyTexSubImage2D(table, save_CopyTexSubImage2D);
SET_TexSubImage1D(table, save_TexSubImage1D);
SET_TexSubImage2D(table, save_TexSubImage2D);
SET_BindTexture(table, save_BindTexture);
SET_PrioritizeTextures(table, save_PrioritizeTextures);
SET_BlendColor(table, save_BlendColor);
SET_BlendEquation(table, save_BlendEquation);
SET_DrawRangeElements(table, save_DrawRangeElements);
SET_TexImage3D(table, save_TexImage3D);
SET_TexSubImage3D(table, save_TexSubImage3D);
SET_CopyTexSubImage3D(table, save_CopyTexSubImage3D);
SET_ActiveTexture(table, save_ActiveTexture);
SET_LoadTransposeMatrixf(table, save_LoadTransposeMatrixf);
SET_LoadTransposeMatrixd(table, save_LoadTransposeMatrixd);
SET_MultTransposeMatrixf(table, save_MultTransposeMatrixf);
SET_MultTransposeMatrixd(table, save_MultTransposeMatrixd);
SET_SampleCoverage(table, save_SampleCoverage);
SET_CompressedTexImage3D(table, save_CompressedTexImage3D);
SET_CompressedTexImage2D(table, save_CompressedTexImage2D);
SET_CompressedTexImage1D(table, save_CompressedTexImage1D);
SET_CompressedTexSubImage3D(table, save_CompressedTexSubImage3D);
SET_CompressedTexSubImage2D(table, save_CompressedTexSubImage2D);
SET_CompressedTexSubImage1D(table, save_CompressedTexSubImage1D);
SET_BlendFuncSeparate(table, save_BlendFuncSeparate);
SET_MultiDrawArrays(table, save_MultiDrawArrays);
SET_MultiDrawElementsEXT(table, save_MultiDrawElementsEXT);
SET_PointParameterf(table, save_PointParameterf);
SET_PointParameterfv(table, save_PointParameterfv);
SET_PointParameteri(table, save_PointParameteri);
SET_PointParameteriv(table, save_PointParameteriv);
SET_WindowPos2d(table, save_WindowPos2d);
SET_WindowPos2dv(table, save_WindowPos2dv);
SET_WindowPos2f(table, save_WindowPos2f);
SET_WindowPos2fv(table, save_WindowPos2fv);
SET_WindowPos2i(table, save_WindowPos2i);
SET_WindowPos2iv(table, save_WindowPos2iv);
SET_WindowPos2s(table, save_WindowPos2s);
SET_WindowPos2sv(table, save_WindowPos2sv);
SET_WindowPos3d(table, save_WindowPos3d);
SET_WindowPos3dv(table, save_WindowPos3dv);
SET_WindowPos3f(table, save_WindowPos3f);
SET_WindowPos3fv(table, save_WindowPos3fv);
SET_WindowPos3i(table, save_WindowPos3i);
SET_WindowPos3iv(table, save_WindowPos3iv);
SET_WindowPos3s(table, save_WindowPos3s);
SET_WindowPos3sv(table, save_WindowPos3sv);
SET_BeginQuery(table, save_BeginQuery);
SET_EndQuery(table, save_EndQuery);
SET_BlendEquationSeparate(table, save_BlendEquationSeparate);
SET_DrawBuffers(table, save_DrawBuffers);
SET_StencilFuncSeparate(table, save_StencilFuncSeparate);
SET_StencilOpSeparate(table, save_StencilOpSeparate);
SET_StencilMaskSeparate(table, save_StencilMaskSeparate);
SET_UseProgram(table, save_UseProgram);
SET_Uniform1f(table, save_Uniform1f);
SET_Uniform2f(table, save_Uniform2f);
SET_Uniform3f(table, save_Uniform3f);
SET_Uniform4f(table, save_Uniform4f);
SET_Uniform1i(table, save_Uniform1i);
SET_Uniform2i(table, save_Uniform2i);
SET_Uniform3i(table, save_Uniform3i);
SET_Uniform4i(table, save_Uniform4i);
SET_Uniform1fv(table, save_Uniform1fv);
SET_Uniform2fv(table, save_Uniform2fv);
SET_Uniform3fv(table, save_Uniform3fv);
SET_Uniform4fv(table, save_Uniform4fv);
SET_Uniform1iv(table, save_Uniform1iv);
SET_Uniform2iv(table, save_Uniform2iv);
SET_Uniform3iv(table, save_Uniform3iv);
SET_Uniform4iv(table, save_Uniform4iv);
SET_UniformMatrix2fv(table, save_UniformMatrix2fv);
SET_UniformMatrix3fv(table, save_UniformMatrix3fv);
SET_UniformMatrix4fv(table, save_UniformMatrix4fv);
SET_UniformMatrix2x3fv(table, save_UniformMatrix2x3fv);
SET_UniformMatrix3x2fv(table, save_UniformMatrix3x2fv);
SET_UniformMatrix2x4fv(table, save_UniformMatrix2x4fv);
SET_UniformMatrix4x2fv(table, save_UniformMatrix4x2fv);
SET_UniformMatrix3x4fv(table, save_UniformMatrix3x4fv);
SET_UniformMatrix4x3fv(table, save_UniformMatrix4x3fv);
SET_ProgramStringARB(table, save_ProgramStringARB);
SET_BindProgramARB(table, save_BindProgramARB);
SET_ProgramEnvParameter4dARB(table, save_ProgramEnvParameter4dARB);
SET_ProgramEnvParameter4dvARB(table, save_ProgramEnvParameter4dvARB);
SET_ProgramEnvParameter4fARB(table, save_ProgramEnvParameter4fARB);
SET_ProgramEnvParameter4fvARB(table, save_ProgramEnvParameter4fvARB);
SET_ProgramLocalParameter4dARB(table, save_ProgramLocalParameter4dARB);
SET_ProgramLocalParameter4dvARB(table, save_ProgramLocalParameter4dvARB);
SET_ProgramLocalParameter4fARB(table, save_ProgramLocalParameter4fARB);
SET_ProgramLocalParameter4fvARB(table, save_ProgramLocalParameter4fvARB);
SET_ClampColor(table, save_ClampColor);
SET_DrawArraysInstancedARB(table, save_DrawArraysInstancedARB);
SET_DrawElementsInstancedARB(table, save_DrawElementsInstancedARB);
SET_BlitFramebuffer(table, save_BlitFramebuffer);
SET_VertexAttribDivisor(table, save_VertexAttribDivisor);
SET_UniformBlockBinding(table, save_UniformBlockBinding);
SET_DrawElementsBaseVertex(table, save_DrawElementsBaseVertex);
SET_MultiDrawElementsBaseVertex(table, save_MultiDrawElementsBaseVertex);
SET_DrawElementsInstancedBaseVertex(table, save_DrawElementsInstancedBaseVertex);
SET_WaitSync(table, save_WaitSync);
SET_BlendEquationiARB(table, save_BlendEquationiARB);
SET_BlendEquationSeparateiARB(table, save_BlendEquationSeparateiARB);
SET_BlendFunciARB(table, save_BlendFunciARB);
SET_BlendFuncSeparateiARB(table, save_BlendFuncSeparateiARB);
SET_BindSampler(table, save_BindSampler);
SET_SamplerParameteri(table, save_SamplerParameteri);
SET_SamplerParameterf(table, save_SamplerParameterf);
SET_SamplerParameteriv(table, save_SamplerParameteriv);
SET_SamplerParameterfv(table, save_SamplerParameterfv);
SET_SamplerParameterIiv(table, save_SamplerParameterIiv);
SET_SamplerParameterIuiv(table, save_SamplerParameterIuiv);
SET_QueryCounter(table, save_QueryCounter);
SET_UniformSubroutinesuiv(table, save_UniformSubroutinesuiv);
SET_PatchParameteri(table, save_PatchParameteri);
SET_PatchParameterfv(table, save_PatchParameterfv);
SET_DrawArraysIndirect(table, save_DrawArraysIndirect);
SET_DrawElementsIndirect(table, save_DrawElementsIndirect);
SET_MultiDrawArraysIndirect(table, save_MultiDrawArraysIndirect);
SET_MultiDrawElementsIndirect(table, save_MultiDrawElementsIndirect);
SET_Uniform1d(table, save_Uniform1d);
SET_Uniform2d(table, save_Uniform2d);
SET_Uniform3d(table, save_Uniform3d);
SET_Uniform4d(table, save_Uniform4d);
SET_Uniform1dv(table, save_Uniform1dv);
SET_Uniform2dv(table, save_Uniform2dv);
SET_Uniform3dv(table, save_Uniform3dv);
SET_Uniform4dv(table, save_Uniform4dv);
SET_UniformMatrix2dv(table, save_UniformMatrix2dv);
SET_UniformMatrix3dv(table, save_UniformMatrix3dv);
SET_UniformMatrix4dv(table, save_UniformMatrix4dv);
SET_UniformMatrix2x3dv(table, save_UniformMatrix2x3dv);
SET_UniformMatrix2x4dv(table, save_UniformMatrix2x4dv);
SET_UniformMatrix3x2dv(table, save_UniformMatrix3x2dv);
SET_UniformMatrix3x4dv(table, save_UniformMatrix3x4dv);
SET_UniformMatrix4x2dv(table, save_UniformMatrix4x2dv);
SET_UniformMatrix4x3dv(table, save_UniformMatrix4x3dv);
SET_ProgramUniform1d(table, save_ProgramUniform1d);
SET_ProgramUniform2d(table, save_ProgramUniform2d);
SET_ProgramUniform3d(table, save_ProgramUniform3d);
SET_ProgramUniform4d(table, save_ProgramUniform4d);
SET_ProgramUniform1dv(table, save_ProgramUniform1dv);
SET_ProgramUniform2dv(table, save_ProgramUniform2dv);
SET_ProgramUniform3dv(table, save_ProgramUniform3dv);
SET_ProgramUniform4dv(table, save_ProgramUniform4dv);
SET_ProgramUniformMatrix2dv(table, save_ProgramUniformMatrix2dv);
SET_ProgramUniformMatrix3dv(table, save_ProgramUniformMatrix3dv);
SET_ProgramUniformMatrix4dv(table, save_ProgramUniformMatrix4dv);
SET_ProgramUniformMatrix2x3dv(table, save_ProgramUniformMatrix2x3dv);
SET_ProgramUniformMatrix2x4dv(table, save_ProgramUniformMatrix2x4dv);
SET_ProgramUniformMatrix3x2dv(table, save_ProgramUniformMatrix3x2dv);
SET_ProgramUniformMatrix3x4dv(table, save_ProgramUniformMatrix3x4dv);
SET_ProgramUniformMatrix4x2dv(table, save_ProgramUniformMatrix4x2dv);
SET_ProgramUniformMatrix4x3dv(table, save_ProgramUniformMatrix4x3dv);
SET_DrawTransformFeedbackStream(table, save_DrawTransformFeedbackStream);
SET_BeginQueryIndexed(table, save_BeginQueryIndexed);
SET_EndQueryIndexed(table, save_EndQueryIndexed);
SET_UseProgramStages(table, save_UseProgramStages);
SET_ProgramUniform1i(table, save_ProgramUniform1i);
SET_ProgramUniform2i(table, save_ProgramUniform2i);
SET_ProgramUniform3i(table, save_ProgramUniform3i);
SET_ProgramUniform4i(table, save_ProgramUniform4i);
SET_ProgramUniform1ui(table, save_ProgramUniform1ui);
SET_ProgramUniform2ui(table, save_ProgramUniform2ui);
SET_ProgramUniform3ui(table, save_ProgramUniform3ui);
SET_ProgramUniform4ui(table, save_ProgramUniform4ui);
SET_ProgramUniform1f(table, save_ProgramUniform1f);
SET_ProgramUniform2f(table, save_ProgramUniform2f);
SET_ProgramUniform3f(table, save_ProgramUniform3f);
SET_ProgramUniform4f(table, save_ProgramUniform4f);
SET_ProgramUniform1iv(table, save_ProgramUniform1iv);
SET_ProgramUniform2iv(table, save_ProgramUniform2iv);
SET_ProgramUniform3iv(table, save_ProgramUniform3iv);
SET_ProgramUniform4iv(table, save_ProgramUniform4iv);
SET_ProgramUniform1uiv(table, save_ProgramUniform1uiv);
SET_ProgramUniform2uiv(table, save_ProgramUniform2uiv);
SET_ProgramUniform3uiv(table, save_ProgramUniform3uiv);
SET_ProgramUniform4uiv(table, save_ProgramUniform4uiv);
SET_ProgramUniform1fv(table, save_ProgramUniform1fv);
SET_ProgramUniform2fv(table, save_ProgramUniform2fv);
SET_ProgramUniform3fv(table, save_ProgramUniform3fv);
SET_ProgramUniform4fv(table, save_ProgramUniform4fv);
SET_ProgramUniformMatrix2fv(table, save_ProgramUniformMatrix2fv);
SET_ProgramUniformMatrix3fv(table, save_ProgramUniformMatrix3fv);
SET_ProgramUniformMatrix4fv(table, save_ProgramUniformMatrix4fv);
SET_ProgramUniformMatrix2x3fv(table, save_ProgramUniformMatrix2x3fv);
SET_ProgramUniformMatrix3x2fv(table, save_ProgramUniformMatrix3x2fv);
SET_ProgramUniformMatrix2x4fv(table, save_ProgramUniformMatrix2x4fv);
SET_ProgramUniformMatrix4x2fv(table, save_ProgramUniformMatrix4x2fv);
SET_ProgramUniformMatrix3x4fv(table, save_ProgramUniformMatrix3x4fv);
SET_ProgramUniformMatrix4x3fv(table, save_ProgramUniformMatrix4x3fv);
SET_DrawArraysInstancedBaseInstance(table, save_DrawArraysInstancedBaseInstance);
SET_DrawElementsInstancedBaseInstance(table, save_DrawElementsInstancedBaseInstance);
SET_DrawElementsInstancedBaseVertexBaseInstance(table, save_DrawElementsInstancedBaseVertexBaseInstance);
SET_DrawTransformFeedbackInstanced(table, save_DrawTransformFeedbackInstanced);
SET_DrawTransformFeedbackStreamInstanced(table, save_DrawTransformFeedbackStreamInstanced);
SET_DispatchCompute(table, save_DispatchCompute);
SET_DispatchComputeIndirect(table, save_DispatchComputeIndirect);
SET_ClipControl(table, save_ClipControl);
SET_TextureBarrierNV(table, save_TextureBarrierNV);
SET_PrimitiveBoundingBox(table, save_PrimitiveBoundingBox);
SET_Uniform1i64ARB(table, save_Uniform1i64ARB);
SET_Uniform2i64ARB(table, save_Uniform2i64ARB);
SET_Uniform3i64ARB(table, save_Uniform3i64ARB);
SET_Uniform4i64ARB(table, save_Uniform4i64ARB);
SET_Uniform1i64vARB(table, save_Uniform1i64vARB);
SET_Uniform2i64vARB(table, save_Uniform2i64vARB);
SET_Uniform3i64vARB(table, save_Uniform3i64vARB);
SET_Uniform4i64vARB(table, save_Uniform4i64vARB);
SET_Uniform1ui64ARB(table, save_Uniform1ui64ARB);
SET_Uniform2ui64ARB(table, save_Uniform2ui64ARB);
SET_Uniform3ui64ARB(table, save_Uniform3ui64ARB);
SET_Uniform4ui64ARB(table, save_Uniform4ui64ARB);
SET_Uniform1ui64vARB(table, save_Uniform1ui64vARB);
SET_Uniform2ui64vARB(table, save_Uniform2ui64vARB);
SET_Uniform3ui64vARB(table, save_Uniform3ui64vARB);
SET_Uniform4ui64vARB(table, save_Uniform4ui64vARB);
SET_ProgramUniform1i64ARB(table, save_ProgramUniform1i64ARB);
SET_ProgramUniform2i64ARB(table, save_ProgramUniform2i64ARB);
SET_ProgramUniform3i64ARB(table, save_ProgramUniform3i64ARB);
SET_ProgramUniform4i64ARB(table, save_ProgramUniform4i64ARB);
SET_ProgramUniform1i64vARB(table, save_ProgramUniform1i64vARB);
SET_ProgramUniform2i64vARB(table, save_ProgramUniform2i64vARB);
SET_ProgramUniform3i64vARB(table, save_ProgramUniform3i64vARB);
SET_ProgramUniform4i64vARB(table, save_ProgramUniform4i64vARB);
SET_ProgramUniform1ui64ARB(table, save_ProgramUniform1ui64ARB);
SET_ProgramUniform2ui64ARB(table, save_ProgramUniform2ui64ARB);
SET_ProgramUniform3ui64ARB(table, save_ProgramUniform3ui64ARB);
SET_ProgramUniform4ui64ARB(table, save_ProgramUniform4ui64ARB);
SET_ProgramUniform1ui64vARB(table, save_ProgramUniform1ui64vARB);
SET_ProgramUniform2ui64vARB(table, save_ProgramUniform2ui64vARB);
SET_ProgramUniform3ui64vARB(table, save_ProgramUniform3ui64vARB);
SET_ProgramUniform4ui64vARB(table, save_ProgramUniform4ui64vARB);
SET_ViewportArrayv(table, save_ViewportArrayv);
SET_ViewportIndexedf(table, save_ViewportIndexedf);
SET_ViewportIndexedfv(table, save_ViewportIndexedfv);
SET_ScissorArrayv(table, save_ScissorArrayv);
SET_ScissorIndexed(table, save_ScissorIndexed);
SET_ScissorIndexedv(table, save_ScissorIndexedv);
SET_DepthRangeArrayv(table, save_DepthRangeArrayv);
SET_DepthRangeIndexed(table, save_DepthRangeIndexed);
SET_WindowPos4dMESA(table, save_WindowPos4dMESA);
SET_WindowPos4dvMESA(table, save_WindowPos4dvMESA);
SET_WindowPos4fMESA(table, save_WindowPos4fMESA);
SET_WindowPos4fvMESA(table, save_WindowPos4fvMESA);
SET_WindowPos4iMESA(table, save_WindowPos4iMESA);
SET_WindowPos4ivMESA(table, save_WindowPos4ivMESA);
SET_WindowPos4sMESA(table, save_WindowPos4sMESA);
SET_WindowPos4svMESA(table, save_WindowPos4svMESA);
SET_BindFragmentShaderATI(table, save_BindFragmentShaderATI);
SET_SetFragmentShaderConstantATI(table, save_SetFragmentShaderConstantATI);
SET_ActiveStencilFaceEXT(table, save_ActiveStencilFaceEXT);
SET_DepthBoundsEXT(table, save_DepthBoundsEXT);
SET_ProvokingVertex(table, save_ProvokingVertex);
SET_ColorMaski(table, save_ColorMaski);
SET_Enablei(table, save_Enablei);
SET_Disablei(table, save_Disablei);
SET_MatrixLoadfEXT(table, save_MatrixLoadfEXT);
SET_MatrixLoaddEXT(table, save_MatrixLoaddEXT);
SET_MatrixMultfEXT(table, save_MatrixMultfEXT);
SET_MatrixMultdEXT(table, save_MatrixMultdEXT);
SET_MatrixLoadIdentityEXT(table, save_MatrixLoadIdentityEXT);
SET_MatrixRotatefEXT(table, save_MatrixRotatefEXT);
SET_MatrixRotatedEXT(table, save_MatrixRotatedEXT);
SET_MatrixScalefEXT(table, save_MatrixScalefEXT);
SET_MatrixScaledEXT(table, save_MatrixScaledEXT);
SET_MatrixTranslatefEXT(table, save_MatrixTranslatefEXT);
SET_MatrixTranslatedEXT(table, save_MatrixTranslatedEXT);
SET_MatrixOrthoEXT(table, save_MatrixOrthoEXT);
SET_MatrixFrustumEXT(table, save_MatrixFrustumEXT);
SET_MatrixPushEXT(table, save_MatrixPushEXT);
SET_MatrixPopEXT(table, save_MatrixPopEXT);
SET_TextureParameteriEXT(table, save_TextureParameteriEXT);
SET_TextureParameterivEXT(table, save_TextureParameterivEXT);
SET_TextureParameterfEXT(table, save_TextureParameterfEXT);
SET_TextureParameterfvEXT(table, save_TextureParameterfvEXT);
SET_TextureImage1DEXT(table, save_TextureImage1DEXT);
SET_TextureImage2DEXT(table, save_TextureImage2DEXT);
SET_TextureImage3DEXT(table, save_TextureImage3DEXT);
SET_TextureSubImage1DEXT(table, save_TextureSubImage1DEXT);
SET_TextureSubImage2DEXT(table, save_TextureSubImage2DEXT);
SET_TextureSubImage3DEXT(table, save_TextureSubImage3DEXT);
SET_CopyTextureImage1DEXT(table, save_CopyTextureImage1DEXT);
SET_CopyTextureImage2DEXT(table, save_CopyTextureImage2DEXT);
SET_CopyTextureSubImage1DEXT(table, save_CopyTextureSubImage1DEXT);
SET_CopyTextureSubImage2DEXT(table, save_CopyTextureSubImage2DEXT);
SET_CopyTextureSubImage3DEXT(table, save_CopyTextureSubImage3DEXT);
SET_BindMultiTextureEXT(table, save_BindMultiTextureEXT);
SET_MultiTexEnviEXT(table, save_MultiTexEnviEXT);
SET_MultiTexEnvivEXT(table, save_MultiTexEnvivEXT);
SET_MultiTexEnvfEXT(table, save_MultiTexEnvfEXT);
SET_MultiTexEnvfvEXT(table, save_MultiTexEnvfvEXT);
SET_MultiTexParameteriEXT(table, save_MultiTexParameteriEXT);
SET_MultiTexParameterivEXT(table, save_MultiTexParameterivEXT);
SET_MultiTexParameterfEXT(table, save_MultiTexParameterfEXT);
SET_MultiTexParameterfvEXT(table, save_MultiTexParameterfvEXT);
SET_MultiTexImage1DEXT(table, save_MultiTexImage1DEXT);
SET_MultiTexImage2DEXT(table, save_MultiTexImage2DEXT);
SET_MultiTexImage3DEXT(table, save_MultiTexImage3DEXT);
SET_MultiTexSubImage1DEXT(table, save_MultiTexSubImage1DEXT);
SET_MultiTexSubImage2DEXT(table, save_MultiTexSubImage2DEXT);
SET_MultiTexSubImage3DEXT(table, save_MultiTexSubImage3DEXT);
SET_CopyMultiTexImage1DEXT(table, save_CopyMultiTexImage1DEXT);
SET_CopyMultiTexImage2DEXT(table, save_CopyMultiTexImage2DEXT);
SET_CopyMultiTexSubImage1DEXT(table, save_CopyMultiTexSubImage1DEXT);
SET_CopyMultiTexSubImage2DEXT(table, save_CopyMultiTexSubImage2DEXT);
SET_CopyMultiTexSubImage3DEXT(table, save_CopyMultiTexSubImage3DEXT);
SET_MatrixLoadTransposefEXT(table, save_MatrixLoadTransposefEXT);
SET_MatrixLoadTransposedEXT(table, save_MatrixLoadTransposedEXT);
SET_MatrixMultTransposefEXT(table, save_MatrixMultTransposefEXT);
SET_MatrixMultTransposedEXT(table, save_MatrixMultTransposedEXT);
SET_CompressedTextureImage1DEXT(table, save_CompressedTextureImage1DEXT);
SET_CompressedTextureImage2DEXT(table, save_CompressedTextureImage2DEXT);
SET_CompressedTextureImage3DEXT(table, save_CompressedTextureImage3DEXT);
SET_CompressedTextureSubImage1DEXT(table, save_CompressedTextureSubImage1DEXT);
SET_CompressedTextureSubImage2DEXT(table, save_CompressedTextureSubImage2DEXT);
SET_CompressedTextureSubImage3DEXT(table, save_CompressedTextureSubImage3DEXT);
SET_CompressedMultiTexImage1DEXT(table, save_CompressedMultiTexImage1DEXT);
SET_CompressedMultiTexImage2DEXT(table, save_CompressedMultiTexImage2DEXT);
SET_CompressedMultiTexImage3DEXT(table, save_CompressedMultiTexImage3DEXT);
SET_CompressedMultiTexSubImage1DEXT(table, save_CompressedMultiTexSubImage1DEXT);
SET_CompressedMultiTexSubImage2DEXT(table, save_CompressedMultiTexSubImage2DEXT);
SET_CompressedMultiTexSubImage3DEXT(table, save_CompressedMultiTexSubImage3DEXT);
SET_NamedProgramStringEXT(table, save_NamedProgramStringEXT);
SET_NamedProgramLocalParameter4fEXT(table, save_NamedProgramLocalParameter4fEXT);
SET_NamedProgramLocalParameter4fvEXT(table, save_NamedProgramLocalParameter4fvEXT);
SET_NamedProgramLocalParameter4dEXT(table, save_NamedProgramLocalParameter4dEXT);
SET_NamedProgramLocalParameter4dvEXT(table, save_NamedProgramLocalParameter4dvEXT);
SET_TextureParameterIivEXT(table, save_TextureParameterIivEXT);
SET_TextureParameterIuivEXT(table, save_TextureParameterIuivEXT);
SET_MultiTexParameterIivEXT(table, save_MultiTexParameterIivEXT);
SET_MultiTexParameterIuivEXT(table, save_MultiTexParameterIuivEXT);
SET_SubpixelPrecisionBiasNV(table, save_SubpixelPrecisionBiasNV);
SET_ConservativeRasterParameterfNV(table, save_ConservativeRasterParameterfNV);
SET_ConservativeRasterParameteriNV(table, save_ConservativeRasterParameteriNV);
SET_PolygonOffsetClampEXT(table, save_PolygonOffsetClampEXT);
SET_WindowRectanglesEXT(table, save_WindowRectanglesEXT);
SET_StencilFuncSeparateATI(table, save_StencilFuncSeparateATI);
SET_ProgramEnvParameters4fvEXT(table, save_ProgramEnvParameters4fvEXT);
SET_ProgramLocalParameters4fvEXT(table, save_ProgramLocalParameters4fvEXT);
SET_ClearColorIiEXT(table, save_ClearColorIiEXT);
SET_ClearColorIuiEXT(table, save_ClearColorIuiEXT);
SET_TexParameterIiv(table, save_TexParameterIiv);
SET_TexParameterIuiv(table, save_TexParameterIuiv);
SET_Uniform1ui(table, save_Uniform1ui);
SET_Uniform2ui(table, save_Uniform2ui);
SET_Uniform3ui(table, save_Uniform3ui);
SET_Uniform4ui(table, save_Uniform4ui);
SET_Uniform1uiv(table, save_Uniform1uiv);
SET_Uniform2uiv(table, save_Uniform2uiv);
SET_Uniform3uiv(table, save_Uniform3uiv);
SET_Uniform4uiv(table, save_Uniform4uiv);
SET_ClearBufferiv(table, save_ClearBufferiv);
SET_ClearBufferuiv(table, save_ClearBufferuiv);
SET_ClearBufferfv(table, save_ClearBufferfv);
SET_ClearBufferfi(table, save_ClearBufferfi);
SET_BeginTransformFeedback(table, save_BeginTransformFeedback);
SET_EndTransformFeedback(table, save_EndTransformFeedback);
SET_BeginConditionalRender(table, save_BeginConditionalRender);
SET_EndConditionalRender(table, save_EndConditionalRender);
SET_BindTransformFeedback(table, save_BindTransformFeedback);
SET_PauseTransformFeedback(table, save_PauseTransformFeedback);
SET_ResumeTransformFeedback(table, save_ResumeTransformFeedback);
SET_DrawTransformFeedback(table, save_DrawTransformFeedback);
