/**************************************************************************
 *
 * Copyright 2013 VMware, Inc.
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL VMWARE AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **************************************************************************/

#ifndef TR_DUMP_DEFINES_H_
#define TR_DUMP_DEFINES_H_

#include "pipe/p_compiler.h"
#include "util/format/u_format.h"
#include "util/u_dump.h"
#include "tr_dump.h"


static inline void
trace_dump_format(enum pipe_format format)
{
   if (!trace_dumping_enabled_locked())
      return;

   trace_dump_enum(util_format_name(format));
}


static inline void
trace_dump_query_type(unsigned value)
{
   if (!trace_dumping_enabled_locked())
      return;

   trace_dump_enum(util_str_query_type(value, false));
}

static inline void
trace_dump_query_flags(unsigned value)
{
   if (!trace_dumping_enabled_locked())
      return;

   trace_dump_enum(util_str_query_type(value, false));
}



#endif /* TR_DUMP_DEFINES_H_ */
